clc; clear; format compact; format long g;
% vypocet pomoci "Mapping Toolbox 2.2"

%% ZADANI [m, deg] (behem reseni jsou vsechny hodnoty v radianech) %%%%%%%%
% polomer zeme
R = 6378000; % [m]
% data pro 1. ortodromu/loxodromu, fiA, lambdaA, alfaA
K1 = [ dms2rad([76 48 53.79]) dms2rad([63 36 01.44]) dms2rad([134 27 18.86]) ];
% data pro 2. ortodromu/loxodromu, fiB, lambdaB; fiC, lambdaC
K2 = [ dms2rad([78 54 56.69]) dms2rad([291 09 55.44]) dms2rad(-[55 34 41.67]) dms2rad([92 55 09.77]) ];
% behem reseni jsou vsechny hodnoty v radianech

%% CO ZOBRAZIT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gnomonicka = 0; % ortodromy primky
mercatorovo = 0; % loxodromy primky
stred = [90 0 0]; % stred zobrazovaneho uzemi [fi lambda orientace] pro gnomonickou projekci
ortodromy = 1;
loxodromy = 0;
color = 'y'; % barva ortodrom / loxodrom;
% u loxodrom zmenit azimut o 180, pokud se nevykresli cela (dole VIZUALIZACE)

%% Uloha 1.2 - PRVNI Ortodroma %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% PRUSECIK NA ROVNIKU (oba)
rov1 = gcxgc(K1(1), K1(2), K1(3),  0, 0, pi/2,  'rad');
rov1 = rov1(3:4); % [lambda1, lambda2]
Arov1 = azimuth(0, rov1,  K1(1), K1(2),  'rad'); % [azimut1, azimut2]

% PRUSECIK S 0 POLEDNIKEM
nul1 = gcxgc(K1(1), K1(2), K1(3),  0, 0, 0,  'rad');
if abs(nul1(3)-0) < 1e-10 % [lambda], vylouceni pruseciku s 180 polednikem
    nul1 = nul1(1); % [fi]
else
    nul1 = nul1(2);
end
Anul1 = azimuth(nul1, 0,  K1(1), K1(2),  'rad'); % [azimut]

% NEJSEVERNEJSI BOD
if nul2pi( Arov1(1) ) < pi/2
    sev1(1) = pi/2 - nul2pi( Arov1(1) ); % [fi]
    sev1(2) = pi/2 + rov1(1); % [lambda]
else
    sev1(1) = pi/2 - nul2pi( Arov1(2) ); % [fi]
    sev1(2) = pi/2 + rov1(2); % [lambda]
end

% OBRATNIK RAKA
rak1 = gcxsc(K1(1), K1(2), K1(3),  pi/2, 0, 66.5*pi/180,  'rad'); % parametry rovnobezky: sirka stredu, delka stredu, polomer (doplnek do 90)
nrak1 = size(rak1,2) /2; % pocet pruseciku
for i = 1:nrak1
    Arak1(i) = azimuth(rak1(i), rak1(i+2),  K1(1), K1(2),  'rad'); % [azimut1, azimut2]
end
if nrak1 == 2
    rak1 = rak1(3:4);
elseif nrak1 == 1
    rak1 = rak1(2);
end

%% Uloha 1.2 - DRUHA Ortodroma %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% azimut z B na C
azBC = azimuth(K2(1), K2(2),  K2(3), K2(4),  'rad');

% PRUSECIK NA ROVNIKU (oba)
rov2 = gcxgc(K2(1), K2(2), azBC,  0, 0, pi/2,  'rad');
rov2 = rov2(3:4); % [lambda1, lambda2]
Arov2 = azimuth(0, rov2,  K2(1), K2(2),  'rad'); % [azimut1, azimut2]

% PRUSECIK S 0 POLEDNIKEM
nul2 = gcxgc(K2(1), K2(2), azBC,  0, 0, 0,  'rad');
if abs(nul2(3)-0) < 1e-10 % [lambda], vylouceni pruseciku s 180 polednikem
    nul2 = nul2(1); % [fi]
else
    nul2 = nul2(2);
end
Anul2 = azimuth(nul2, 0,  K2(1), K2(2),  'rad'); % [azimut]

% NEJSEVERNEJSI BOD
if nul2pi( Arov2(1) ) < pi/2
    sev2(1) = pi/2 - nul2pi( Arov2(1) ); % [fi]
    sev2(2) = pi/2 + rov2(1); % [lambda]
else
    sev2(1) = pi/2 - nul2pi( Arov2(2) ); % [fi]
    sev2(2) = pi/2 + rov2(2); % [lambda]
end

% OBRATNIK RAKA
rak2 = gcxsc(K2(1), K2(2), azBC,  pi/2, 0, 66.5*pi/180,  'rad'); % [fi1, fi2, lambda1, lambda2]
    % parametry rovnobezky: sirka stredu, delka stredu, polomer (doplnek do 90)
nrak2 = size(rak2,2) /2; % pocet pruseciku 
for i = 1:nrak2
    Arak2(i) = azimuth(rak2(i), rak2(i+2),  K2(1), K2(2),  'rad'); % [azimut1, azimut2]
end
if nrak2 == 2
    rak2 = rak2(3:4);
elseif nrak2 == 1
    rak2 = rak2(2);
end

%% PRUSECIK ORTODROM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pr_orto = gcxgc(K1(1), K1(2), K1(3),  K2(1), K2(2), azBC,  'rad'); % [fi1, fi2, lambda1, lambda2]

%% Uloha 1.3 - Loxodroma %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 2. loxodroma (dopocet azimutu pro kratsi reseni)
lox_az = azimuth('rh',K2(1),K2(2),K2(3),K2(4),'rad');
lox_dist = distance('rh',K2(1),K2(2),K2(3),K2(4),'rad');

% prusecik loxodrom
pr_lox = rhxrh(K1(1), K1(2), K1(3),  K2(1), K2(2), lox_az,  'rad');
pr_lox_az = azimuth('rh', K2(1), K2(2),  pr_lox(1), pr_lox(2),  'rad');
pr_lox_dist = distance('rh',K1(1), K1(2),  pr_lox(1), pr_lox(2),  'rad');

%% RESENI PODLE UCEBNIHO TEXTU 
lox_az2 = atan2( K2(4)-K2(2),  log( tan(K2(3)/2 + pi/4) )   -   log( tan(K2(1)/2 + pi/4) )  );
lox_dist2 = 1/cos(lox_az2)*(K2(3)-K2(1));

for k = -100:100
    Vyraz = ( 1/(tan(K1(3)) - tan(lox_az2)) * ( tan(K1(3))*log(tan(K1(1)/2 + pi/4)) - tan(lox_az2) * log(tan(K2(1)/2 + pi/4)) - K1(2) + K2(2) + 2*k*pi) );
    fiX(k+101) = 2*( atan( exp( Vyraz ) ) - pi/4);
end

% nejblizsi fiX
pr_lox2(1) = min(abs(K1(1) - fiX));
pr_lox2(1) = (K1(1) - pr_lox2(1));

% lambdaX
pr_lox2(2) = tan(K1(3))*(log(tan(pr_lox2(1)/2 + pi/4)) - log(tan(K1(1)/2 + pi/4))) + K1(2);

% vzdalenost B - pr_lox2
pr_lox_dist2 = distance('rh', K1(1), K1(2), pr_lox2(1), pr_lox2(2), 'rad');

%% VYSLEDKY

disp('==================================================')
disp('Priklad 1.2 - 1. ortodroma')
disp('Prusecik s rovnikem [lamda1 lambda2 azimut1 azimut2]')
rad2dms( [ nul22pi(rov1) nul2pi(Arov1) ] ); 
disp('Prusecik s nultym polednikem [fi azimut]')
rad2dms( [ nul1 nul2pi(Anul1) ] );
disp('Nejsevernejsi bod [fi lambda]')
rad2dms( [ sev1(1) nul22pi(sev1(2)) ] );
disp('Pruseciky s obratnikem raka [lambda1 lambda2 azimut1 azimut2]')
rad2dms( [ nul22pi(rak1) nul2pi(Arak1) ] );

fprintf('\n==================================================\n')
disp('Priklad 1.2 - 2. ortodroma')
disp('Azimut BC')
rad2dms( azBC );
disp('Prusecik s rovnikem [lamda1 lambda2 azimut1 azimut2]')
rad2dms( [ nul22pi(rov2) nul2pi(Arov2) ] ); 
disp('Prusecik s nultym polednikem [fi azimut]')
rad2dms( [ nul2 nul2pi(Anul2) ] );
disp('Nejsevernejsi bod [fi lambda]')
rad2dms( [ sev2(1) nul22pi(sev2(2)) ] );
disp('Pruseciky s obratnikem raka [lambda1 lambda2 azimut1 azimut2]')
rad2dms( [ nul22pi(rak2) nul2pi(Arak2) ] );

fprintf('\n==================================================\n')
disp('Priklad 1.2 - prusecik ortodrom [fi1 fi2 lambda1 lambda2]')
rad2dms( [ pr_orto(1:2) nul22pi(pr_orto(3:4)) ] );

fprintf('\n==================================================\n')
disp('Priklad 1.3 - 2. loxodroma')
disp('azimut [], dist [m] (vypocet mapping toolboxem)')
rad2dms( nul2pi(lox_az) );
fprintf( ' |BC| = %.2f m\n', lox_dist*R )
disp('azimut [], dist [m] (vypocet ze vzorce)')
rad2dms( nul2pi(lox_az2) );
fprintf( ' |BC| = %.2f m\n', lox_dist2*R )

fprintf('\n==================================================\n')
disp('Priklad 1.3 - prusecik loxodrom (P) [fi lambda]')
disp('fi, lambda;  |AP|')
rad2dms( [ pr_lox(1) nul22pi(pr_lox(2)) ] );
fprintf( ' |AP| = %.2f m\n', pr_lox_dist*R )

%% VIZUALIZACE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cla
if mercatorovo == 1
    axesm ('mercator','Grid', 'on');
elseif gnomonicka == 1
    axesm ('gnomonic','Grid', 'on', 'Origin',stred);
else
    axesm ('globe','Grid', 'on'); % zobrazeni globu
    view(135,30) % pohled (azimut, elevace)
end
axis off; hold on % vypnuti zobrazeni os; zakaz prepisovani novymi objekty
load topo % nacteni dat povrchu
meshm(topo,topolegend,[90 180]) % zobrazeni povrchu na globus
demcmap(topo) % prirazeni barev vyskam
previewmap % zobrazeni pro tisk

% rovnik
[lat,long] = track1('gc',0,0,90);
plotm(lat,long,'k--','LineWidth',3)
% nulty polednik
[lat,long] = track1('gc',0,0,0,[-90 90]);
plotm(lat,long,'k--','LineWidth',3)
% obratnik raka
if ortodromy == 1
[lat,long] = scircle1(89.9999,0,66.5);
plotm(lat,long,'k--','LineWidth',3)
end

% prevod
K1 = K1 *180/pi;
K2 = K2 *180/pi;
azBC = azBC *180/pi;

%% ORTODROMY
if ortodromy == 1
% ortodroma 1
[lat,long] = track1('gc',K1(1),K1(2),K1(3));
plotm(lat,long,color,'LineWidth',3)
% pruseciky 1. ortodroma
plotm(0, rov1(1)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(0, rov1(2)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(nul1(1)*180/pi, 0,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(sev1(1)*180/pi, sev1(2)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(22.5, rak1(1)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(22.5, rak1(2)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
% ortodroma 2
[lat,long] = track1('gc',K2(1),K2(2), azBC);
plotm(lat,long,color,'LineStyle','--','LineWidth',3)
% pruseciky 2. ortodroma
plotm(0, rov2(1)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(0, rov2(2)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(nul2(1)*180/pi, 0,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(sev2(1)*180/pi, sev2(2)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(22.5, rak2(1)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(22.5, rak2(2)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
% priseciky ortodrom
plotm(pr_orto(1)*180/pi, pr_orto(3)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
plotm(pr_orto(2)*180/pi, pr_orto(4)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
end

%% LOXODROMY
if loxodromy == 1
% loxodroma 1
[lat,long] = track1('rh',K1(1),K1(2),K1(3)); % -180 aby se vykreslila cela
plotm(lat,long,color,'LineWidth',3)
% loxodroma 2
[lat,long] = track1('rh',K2(1),K2(2),lox_az*180/pi);
plotm(lat,long,color,'LineStyle','--','LineWidth',3)
    % % loxodroma 2.2 (vypocet ze vzorce)
    % [lat,long] = track1('rh',K2(1),K2(2),az2*180/pi-180);
    % plotm(lat,long,'y--','LineWidth',3)
% prisecik loxodrom
plotm(pr_lox(1)*180/pi, pr_lox(2)*180/pi,  'rx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','r')
end

%% DANE BODY
plotm(K1(1), K1(2),  'kx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','k')
plotm(K2(1), K2(2),  'kx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','k')
plotm(K2(3), K2(4),  'kx','MarkerSize',10,'LineWidth',3,'MarkerFaceColor','k')
